@@ -1,6 +1,6 @@
 package TPath::Compiler;
 {
-  $TPath::Compiler::VERSION = '0.018';
+  $TPath::Compiler::VERSION = '0.019';
 }
 
 # ABSTRACT: takes ASTs and returns compiled L<TPath::Expression> objects
@@ -15,6 +15,7 @@ use parent 'Exporter';
 
 use aliased 'TPath::Attribute';
 use aliased 'TPath::AttributeTest';
+use aliased 'TPath::Concatenation';
 use aliased 'TPath::Expression';
 use aliased 'TPath::Function';
 use aliased 'TPath::Math';
@@ -363,6 +364,8 @@ sub arg {
     return math( $m, $forester ) if defined $m;
     my $f = $arg->{function};
     return function( $f, $forester ) if defined $f;
+    my $c = $arg->{concat};
+    return concat( $forester, $c ) if defined $c;
     my $op = $arg->{condition}{operator};
     return condition( $arg, $forester, $op ) if defined $op;
     die
@@ -370,6 +373,33 @@ sub arg {
       . ( join ', ', sort keys %$arg );
 }
 
+sub concat {
+    my ( $f, $c ) = @_;
+    my @args = map { arg( $_, $f ) } @{ $c->{carg} };
+
+    # concatenate all constants
+    my $i = 0;
+    my $acc;
+    while ( $i < @args ) {
+        my $v = $args[$i];
+        if ( ref $v ) {
+            $i++;
+            $acc = 0;
+        }
+        elsif ($acc) {
+            $args[ $i - 1 ] .= $v;
+            splice @args, $i, 1;
+            $acc = 1;
+        }
+        else {
+            $acc = 1;
+            $i++;
+        }
+    }
+    return $args[0] unless @args > 1;
+    return Concatenation->new( args => \@args );
+}
+
 sub function {
     my ( $f, $forester ) = @_;
     my $name = $f->{f};
@@ -436,7 +466,7 @@ TPath::Compiler - takes ASTs and returns compiled L<TPath::Expression> objects
 
 =head1 VERSION
 
-version 0.018
+version 0.019
 
 =head1 DESCRIPTION
 
