@@ -2,7 +2,7 @@
 
 package TPath::Grammar;
 {
-  $TPath::Grammar::VERSION = '0.018';
+  $TPath::Grammar::VERSION = '0.019';
 }
 
 use v5.10;
@@ -154,7 +154,13 @@ our $path_grammar = do {
        <rule: args> \( <[arg]> (?: , <[arg]> )* \) <.cp>
     
        <token: arg>
-           <v=literal> | <v=num> | <attribute> | <treepath> | <attribute_test> | <condition>
+           <v=literal> | <v=num> | <concat> | <attribute> | <treepath> | <attribute_test> | <condition>
+       
+       <rule: concat> # string concatenation
+           <[carg]> (?: ~ <[carg]>)+
+       
+       <token: carg>
+           <v=literal> | <v=num> | <attribute> | <treepath> | <math>
     
        <token: num> <.signed_int> | <.float>
     
@@ -202,7 +208,7 @@ our $path_grammar = do {
     
        <token: cmp> (?: [<>=]=?+ | ![=~] | =~ | =?\|= | =\| ) <.cp>
     
-       <token: value> <v=literal> | <v=num> | <attribute> | <treepath> | <math>
+       <token: value> <v=literal> | <v=num> | <concat> | <attribute> | <treepath> | <math>
        
        <rule: math> <function> | <[item=operand]> (?: <.ws> <[item=mop]> <[item=operand]> )*
        
@@ -1331,7 +1337,7 @@ TPath::Grammar - parses TPath expressions into ASTs
 
 =head1 VERSION
 
-version 0.018
+version 0.019
 
 =head1 SYNOPSIS
 
